#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
A tool manager for GIT and/or MAVEN folders

@author: Ualter Otoni Pereira
ualter.junior@gmail.com

"""

import os, sys, configparser, logging, re
import xml.etree.ElementTree as ET
from pathlib import Path
from urllib.request import pathname2url
from tinydb import TinyDB, Query, where
from tinydb.storages import JSONStorage
from tinydb.middlewares import CachingMiddleware
from os.path import expanduser
from pymvngit.pygit import PyGit
from pymvngit.console import Console
from pymvngit.utils import Utils, Colors, Emoticons
from pymvngit.prettytable import PrettyTable
from pymvngit.tableargs import TableArgs

__author__    = "Ualter Otoni Pereira"
__copyright__ = "Ualter Otoni Pereira"
__license__   = "MIT"

_logger = logging.getLogger("app." + __name__)
_logger.setLevel(logging.INFO)

GIT = "git"
MAVEN = "maven"
NAME_DB_MACROS = "macros.json"
USER_DIR = os.path.join(expanduser("~"),".pymvngit")
LOG_DIR  = os.path.join(USER_DIR,"logs")
DB_PATH_PROJECT = os.path.join(USER_DIR,"#.json")
DB_PATH_MACRO  = os.path.join(USER_DIR,NAME_DB_MACROS)
INI_FILE = os.path.join(USER_DIR,"pymvngit.ini")

class PyMvnGit:
    '''
    A Python tool to manage Git and/or Maven folders
    __author
    '''

    def __init__(self):
        self._databaseProject   = None
        self._databaseMacro     = None
        self._configIni         = None
        self._connectedDatabase = False
        self._init()

        self._fileDBProject = DB_PATH_PROJECT.replace("#",self._currentDirEncoded())
        self._fileDBMacro   = DB_PATH_MACRO

    def _init(self):
        if not os.path.exists(USER_DIR):
           os.makedirs(USER_DIR)
        self._initLog()
        self._initIniFile()

    def _initLog(self):
        if not os.path.exists(LOG_DIR):
           os.makedirs(LOG_DIR)

        rootLogger = logging.getLogger()
        logFormatter = logging.Formatter("%(asctime)s [%(threadName)-12.12s] [%(levelname)-5.5s]  %(message)s")   

    def _initDBMacroDefault(self):
        createDefault = False
        if len(self._databaseMacro.all()) == 0:
           createDefault = True

        if createDefault:   
            self._databaseMacro.insert({
                "key":"status",
                "description": "Git Status all projecs",
                "executions":[
                    {
                        "tool":"git",
                        "command":"status",
                        "projects":[]
                    }
                ]
            })

            self._databaseMacro.insert({
                "key":"install_commons",
                "description": "Build and install enrollment client locally",
                "executions":[
                    {
                        "tool":"maven",
                        "command":"clean install",
                        "projects":["teachstore-commons"]
                    },
                    {
                        "tool":"git",
                        "command":"-a push -r commons -b develop -cm $1",
                        "projects":["teachstore-commons"]
                    }
                ]
            })

            self._databaseMacro.insert({
                "key": "all-once",
                "description": "Git Status Add Commit Push ",
                "executions": [
                    {
                        "tool": "git",
                        "command": "status",
                        "projects": ["teachstore-commons","teachstore-enrollment-client"]
                    },
                    {
                        "tool": "git",
                        "command": "add $1",
                        "projects": ["teachstore-commons","teachstore-enrollment-client"]
                    },
                    {
                        "tool": "git",
                        "command": "commit -m $2",
                        "projects": ["teachstore-commons","teachstore-enrollment-client"]
                    },
                    {
                        "tool": "git",
                        "command": "push",
                        "projects": ["teachstore-commons","teachstore-enrollment-client"]
                    },
                    {
                        "tool": "git",
                        "command": "tag -a $3 -m $4",
                        "projects": ["teachstore-commons"]
                    }
                ]
            })


    def _initIniFile(self):
        if os.path.exists(INI_FILE):
           self._configIni = configparser.ConfigParser(allow_no_value=True)
           self._configIni.read(INI_FILE)
        else:
           self._configIni = configparser.ConfigParser(allow_no_value=True) 
           self._configIni.add_section("GENERAL")
           with open(INI_FILE,'w') as c:
               self._configIni.write(c)
           self._configIni = configparser.ConfigParser(allow_no_value=True)
           self._configIni.read(INI_FILE)    

    def _initDBProject(self):
        createIt = True
        if os.path.exists(self._fileDBProject):
           createIt = False
           Console.warning("The repository {}{}{} already exists!".format(Colors.IGREEN,self._currentDir(),Colors.RESET))
           response = input("     {nc}This would recreated it, would you like to proceed ? [{c1}Y{nc}/{c1}N{nc}]: ".format(c1=Colors.IGREEN, nc=Colors.GREEN))
           response = response.upper()
           if "Y" == response:
               self._databaseProject.close()
               self._databaseProject.close()
               os.remove(self._fileDBProject)
               self._connectDBs(startDatabases=True)
           else:
               print("     Ok, bye!\n")
               sys.exit()    
        else:       
           self._connectDBs(startDatabases=True)       

    def _connectDBs(self, startDatabases=False):
        if not os.path.exists(self._fileDBProject) and not startDatabases:
           print("")
           Console.error("{green}There's no repository created for {iblue}{curdir}{green}, please start it using: {iblue}pymvngit start{reset}". \
                         format(curdir=self._fileDBProject,iblue=Colors.IBLUE,igreen=Colors.IGREEN,green=Colors.GREEN,reset=Colors.RESET))
           sys.exit()              

        self._databaseProject = TinyDB(self._fileDBProject, indent=4)
        self._databaseMacro = TinyDB(self._fileDBMacro, indent=4)

        self._initDBMacroDefault()

    def _currentDirEncoded(self):
        nameCurrentDirEncoded = self._encodeDirectoryName(self._currentDir()).replace(":","drive_")
        logging.info("nameCurrentDirEncoded=" + nameCurrentDirEncoded)
        return nameCurrentDirEncoded
    def _currentDir(self):
        currentDirectory = os.getcwd()
        #For testing/development
        #currentDirectory  = "/home/ualter/Developer/teachstore"
        #currentDirectory = "d:\\Developer\\code\\teachstore"
        #currentDirectory  = "/home/ualter/Developer"
        return currentDirectory

    def _encodeDirectoryName(self, directory):
        return directory.replace("\\","_").replace("/","_").replace(":","_drive_").lower()

    def listMacros(self, summary=False, jsonFormat=False):
        """
        \033[0;32mList all the macros available\033[0;0m

        \033[0;32mParameters:\033[0;36m
           jsonFormat (bool): Show in JSON format or a ASCII list

        \033[0;32mReturns:\033[0;36m
           string:Printing results
        \033[0;0m   
        """
        if not self._connectedDatabase:
           self._databaseMacro = TinyDB(self._fileDBMacro, indent=4)

        records = self._databaseMacro.all()
        if jsonFormat:
           Utils.printJSON(Utils.dictToJSON(records))
        else:
           tableArgs   = TableArgs()
           if summary:
              header      = ["#","Key","Description","Executions","Projects"]
              prettyTable = PrettyTable(header)
              idx_lin     = 1
              for r in records:   
                  tools = ""
                  i     = 0
                  for execution in r["executions"]:
                      i += 1
                      tools += "{high}#{i}{co}[{cn}{tool}{co}:{cn}\"{carg}{cmd}{cn}\"{co}]{cn} " \
                               .format(i=i,tool=execution["tool"],cmd=execution["command"], \
                               co=Colors.IBLUE,cn=Colors.GREEN,high=Colors.IGREEN,carg=Colors.CYAN)      
                      projects=", ".join(execution["projects"])
                      if projects == "": projects = "ALL" 
                  
                  columns  = [str(idx_lin), r["key"], r["description"],tools,projects]
                  prettyTable.addRow(columns)
                  idx_lin    += 1
   
              if idx_lin > 1:
                 prettyTable.sortByColumn(0)   
                 prettyTable.numberSeparator = True
                 prettyTable.ascendingOrder(True)
                 Console.printTable("{}MACROS{}".format(Colors.WHITE, Colors.GREEN), prettyTable, tableArgs)    
              else:
                 Console.printLine("No macro found!")
           else:   
              Console.printHeaderCommand("{}MACROS repository: {}".format(Colors.RESET,Colors.IGREEN + self._currentDir() + Colors.GREEN))  
              for r in records:
                  header      = ["Tool/Command","Projects"]
                  prettyTable = PrettyTable(header)
                  idx_lin     = 1
                  i           = 0
                  for execution in r["executions"]:
                      i += 1
                      tool = "{high}#{i}-{co}[{cn}{tool}{co}:{cn}\"{carg}{cmd}{cn}\"{co}]{cn} " \
                               .format(i=i,tool=execution["tool"],cmd=execution["command"], \
                               co=Colors.IBLUE,cn=Colors.GREEN,high=Colors.IGREEN,carg=Colors.CYAN)

                      projects=", ".join(execution["projects"])
                      if projects == "": projects = "ALL" 

                      columns  = [tool, projects]
                      prettyTable.addRow(columns)
                      idx_lin    += 1
                  prettyTable.sortByColumn(0)   
                  prettyTable.numberSeparator = True
                  prettyTable.ascendingOrder(True)
                  Console.printTable2("MACRO: {igreen}{key}{green} - {cyan}{desc}{reset}". \
                      format(key=r["key"],desc=r["description"],igreen=Colors.IGREEN,green=Colors.GREEN,cyan=Colors.CYAN,reset=Colors.RESET), prettyTable, tableArgs)             

        self._databaseMacro.close()
   
    def listProjects(self, jsonFormat=False):
        """
        \033[0;32mList all the projects added to this repository\033[0;0m

        \033[0;32mParameters:\033[0;36m
           jsonFormat (bool): Show in JSON format or a ASCII list

        \033[0;32mReturns:\033[0;36m
           string:Printing Projects
        \033[0;0m   
        """

        for path in Path(USER_DIR).rglob('*.json'):
            if path.name != NAME_DB_MACROS:
               db      = TinyDB(path, indent=4)   
               records = db.all()
               if jsonFormat:
                  Utils.printJSON(Utils.dictToJSON(records))
               else:
                  tableArgs   = TableArgs()
                  header      = ["#","Name","Path","Maven","Git"]
                  prettyTable = PrettyTable(header)
                  idx_lin     = 1
                  for r in records:
                      columns  = [str(idx_lin), r["name"], r["path"], r["maven"], r["git"]]
                      prettyTable.addRow(columns)
                      idx_lin += 1
       
                  prettyTable.sortByColumn(0)   
                  prettyTable.numberSeparator = True
                  prettyTable.ascendingOrder(True)
                  nameDecoded = path.name.replace("_","/").replace("//","/").replace("/drive",":")
                  Console.printTable("FOLDER/REPOSITORY: {}{}{}".format(Colors.IGREEN,nameDecoded,Colors.GREEN), prettyTable, tableArgs)    
               db.close()   

    def clone(self, repositories, localPath, branchName,isRepoListFile):
        args = []
        args.append(repositories)
        args.append(localPath)
        args.append(branchName)
        args.append(isRepoListFile)
        pygit = PyGit("clone",args,None) 
        pygit.execute()

        # Create Repository for the Clone
        self._fileDBProject = DB_PATH_PROJECT.replace("#",self._encodeDirectoryName(localPath))
        self._connectDBs(True)
        if pygit.response:
           for r in pygit.response:
               exist = False if len(self._databaseProject.search(where('name') == r["name"])) == 0 else True
               # Do not include duplicated (most likely the folder was delete but the database entry not)
               if not exist:
                  self._createModule(r["name"],r["path"],r["git"],r["maven"])

    def start(self):
        """
        Creates a repository represeting the Maven/Git projects to be manage together.

        All the subfolder will be analyzed and check if it is a Maven Project and/or a Git Repository. And if it does (one or another),
        it will be added to the PyMvnGit repository.
        """
        self._initDBProject()
        currentDirectory = self._currentDir()

        # Save the current folder as part of the repository
        isMaven = os.path.exists( os.path.join(currentDirectory,"pom.xml") )
        isGit   = os.path.exists( os.path.join(currentDirectory,".git") )
        if isMaven or isGit:
           self._createModule(currentDirectory[currentDirectory.rfind("/")+1:],currentDirectory) 

        # Save each first-level subfolder as part of the repository
        for item in os.listdir(currentDirectory):
            subfolder = os.path.join(currentDirectory, item)
            if not os.path.isfile(subfolder):
               pomPath = os.path.join(currentDirectory, item,"pom.xml")
               gitPath = os.path.join(currentDirectory, item,".git")

               isMaven = os.path.exists(pomPath)
               isGit   = os.path.exists(gitPath)

               if isMaven or isGit:
                  self._createModule(item, subfolder, isMaven, isGit)

    def _createModule(self, folderName, subfolder, isMaven, isGit):
        record = {
            "name":folderName,
            "path":subfolder,
            "git": isGit,
            "maven": isMaven
        }
        self._databaseProject.insert(record)
        #Utils.printJSON(Utils.dictToJSON(record))

    def executeMacro(self, macro):
        """
        \033[0;32mExecute a macro\033[0;0m

        \033[0;32mParameters:\033[0;36m
           macro (string): Key of the macro in the list to be executed

        \033[0;32mReturns:\033[0;36m
           string:Printing results
        \033[0;0m   
        """

        if not self._connectedDatabase:
           self._connectDBs()

        record = self._databaseMacro.search(where('key') == macro)
        if len(record) > 0:
           recordMacro = record[0]

           Console.title2("{c2}EXECUTING MACRO..: {c1}{key}{c2} - {desc}{cn}"\
                             .format(key=recordMacro["key"],desc=recordMacro["description"],\
                              c1=Colors.IGREEN,cn=Colors.RESET,c2=Colors.CYAN),True,False)
           
           tot   = len(recordMacro["executions"])
           index = 0
           for ex in recordMacro["executions"]:
               index += 1

               cmdArgs = self._prepareCommandArguments(ex["tool"], ex["command"])
               cmd     = cmdArgs["command"]
               args    = cmdArgs["arguments"]

               printArgs = "" 
               for i in args:
                   if " " in i.strip(): 
                      printArgs += ("\"" + i + "\"")
                   else:   
                      printArgs += i 

               Console.title("{c2}COMMAND..........: {c1}{cur}{c2} of {c1}{tot}{c2} --> {cmd}"\
                            .format(cur=index,tot=tot,cmd=Colors.ICYAN + ex["tool"] + Colors.CYAN + " " + cmd + " " + printArgs,\
                            c1=Colors.IGREEN,cn=Colors.RESET,c2=Colors.CYAN),True,False)                          
               print("")             

               pNames = ""
               # Selecting the Target Projects
               if len(ex["projects"]) == 0:
                  pNames = "ALL"
                  projects = self._databaseProject.all()
                  #self._databaseProject.search(where("name") == )
               else:
                  projects = []
                  for p in ex["projects"]:
                      if pNames != "": pNames += ", "
                      pNames += p
                      ProjectQ = Query()
                      r = self._databaseProject.search(ProjectQ.name == p)
                      if len(r) > 0:
                         projects.append(r[0])

               if len(projects) == 0:
                  Console.error("Projects [{}] for the macro {} was not found!" \
                                .format(Colors.IGREEN + pNames + Colors.RESET,Colors.IGREEN + macro + Colors.RESET)) 
                  return

               tool = ex["tool"]
               if tool == GIT:
                  pygit = PyGit(cmd,args,projects) 
                  pygit.execute()
                  
           Console.printFooter()       

           Console.printLine("Macro \"{}\" FINISHED!".format(Colors.IYELLOW +  macro + Colors.GREEN),False)
           print("")

        else:
           Console.error("Macro with the key {} was not found!".format(Colors.IGREEN + macro + Colors.RESET))


    def _prepareCommandArguments(self, tool, command):
        result  = {
            "command":command,
            "arguments":""
        }
        command = command.strip()
        if " " not in command:
           return result

        cmd  = command[:command.find(" ")].strip()
        args = command[command.find(" "):].strip()
        args = self._replaceArgsPlaceHolder(tool, command, args)

        result["command"]   = cmd
        result["arguments"] = args

        return result

    def _replaceArgsPlaceHolder(self, tool, originalCmd, args):
        userArguments = {}
        for idx, val in enumerate(sys.argv):
            if idx > 1:
               userArguments[idx-1] = val

        ARG_SEPARATOR = "{{#arg}}"
        isTherePlaceholders = False
        regexp = re.compile(r'\$[0-9]') # Search for $1 $2 $3 $4...
        if regexp.search(args):
           isTherePlaceholders = True

           for i in range(1,10):
               placeHolder = "$"+str(i)
               if placeHolder in args and i in userArguments:
                  args = args.replace(placeHolder,ARG_SEPARATOR + userArguments[i] + ARG_SEPARATOR) 

           if isTherePlaceholders:
              if regexp.search(args):
                 print("")
                 print("  " + Emoticons.error() + " " + Colors.GREEN + "The \033[1;91m{} {}\033[32m contains arguments not passed!".format(tool, originalCmd))
                 print("       Please, check it...")
                 print("")
                 Console.printFooter()
                 sys.exit() 

        argsArray = []
        if args.startswith(ARG_SEPARATOR):
           argsArray.append(args.replace(ARG_SEPARATOR,""))
        else:
           for a in args.split(ARG_SEPARATOR):
               argumentItemCleaned = a.replace(ARG_SEPARATOR,"")
               if len(argumentItemCleaned) > 1:
                  argsArray.append(argumentItemCleaned)
               
        return argsArray 
            
def main():
    pymvngit = PyMvnGit()
    if len(sys.argv) < 2:
       syntax()
       sys.exit()

    command = sys.argv[1]   
    if "help".upper() == command.upper() or "-h".upper() == command.upper():
       syntax()
    elif "listProjects".upper() == command.upper():
       pymvngit.listProjects()
    elif "editMacros".upper() == command.upper() or "editMacro".upper() == command.upper() or "emac".upper() == command.upper():
       pymvngit.editMacros()   
    elif "listMacros".upper() == command.upper():
       summary = True
       if len(sys.argv) > 2 and sys.argv[2] == "-v":
          summary = False
       pymvngit.listMacros(summary)
    elif "start".upper() == command.upper():
       pymvngit.start()
    elif "clone".upper() == command.upper():
       if len(sys.argv) < 6:
          Console.error("Missing parameters! Please, check the syntax of the command clone!")
          syntax(clearScreen=False)
          sys.exit()
       elif len(sys.argv) == 7:
          # Must have 6 or 8 parameters 
          Console.error("Missing parameters! Please, check the syntax of the command clone!")
          syntax(clearScreen=False)
          sys.exit() 
       else:
          repositories   = []
          localPath      = ""
          branchName     = "master" 
          index          = 0
          isRepoListFile = False
          for a in sys.argv:
              index += 1
              if a == "-r":
                 for repo in sys.argv[index].split(","):
                     repositories.append(repo.strip())
              elif a == "-p":
                 localPath = sys.argv[index].strip()
              elif a == "-f":
                 repositories.append(sys.argv[index].strip())
                 isRepoListFile = True
              elif a == "-b":
                 branchName = sys.argv[index].strip()   

          pymvngit.clone(repositories,localPath,branchName,isRepoListFile)       
    else:   
       pymvngit.executeMacro(command)

def syntax(clearScreen=True):
    if clearScreen:
       Console.clear()
    Console.printHeaderCommand("PyGitMvn Tool")    
    print("""
        {green}Tool to manage a group of Git/Maven projects.

        {iblue}MACROS{green}
        ------
        A sequence of Git/Maven commands to run sequentially over project(s) of a repository.

        {iblue}REPOSITORY{green}
        ----------
        A collection of Git/Maven projects that can be managed together.

        {iblue}FUNCTIONS:{green}
        - start...........: {blue}Create a repository from the current directory. Every subfolder will be 
                            analyzed and if it's a Git/Maven project, will be part of this created repository.{green}
        - clone...........: {blue}Create a repository cloning all the URL repository(ies) passed as argument
                            {green}#{cyan}arguments:
                            - Repositories.......: {cyan}(1){green} -r "https://github/projectA, https://github/projectB"{green}
                                                      {cyan}or{green}
                                                   {cyan}(2){green} -f file-name.txt     {cyan}# file with the list of repositories
                            - Local path.........: {green}-p "/home/my/projects"{cyan}
                            - Branch (optional)..: {green}-b "develop"{cyan}             #"branch" is the default value
        - {cyan}[{green}macro{cyan}]{green}.........: {blue}Name of a macro to be executed.{green}
        - listProjects....: {blue}List the projects part of the repository associated with the current directory.{green}
        - listMacros......: {blue}List the macros configured. 
                            {green}#{cyan}arguments:
                            - Show more details, print a table for each one of them..: {green}{{-v}}{cyan}
        {iblue}SYNTAX:{green}
          pymvngit {iblue}[MACRO] {cyan}{{arguments}}
        {iblue}EXAMPLES:{green}
          pymvngit {iblue}build-all{green}
          pymvngit {iblue}add-push{cyan} -A Patch v0.5.7 BugFixCrashing{green}
          pymvngit {iblue}status-all{green}
          pymvngit {iblue}install-commons{green}
          pymvngit {iblue}ltags{green}

    """.format(igreen=Colors.IGREEN,iblue=Colors.IBLUE,ired=Colors.IRED,blue=Colors.BLUE, \
               green=Colors.GREEN,reset=Colors.RESET,cyan=Colors.CYAN,icyan=Colors.ICYAN))

if __name__ == '__main__':
    main()

    """
    {
        "key":"install_commons",
        "description": "Build and install enrollment client locally",
        "commands":[
            {
                "command":"maven",
                "arguments":["clean","install"],
                "projects":["teachstore-commons"]
            },
            {
                "command":"git",
                "arguments":"-a push -r commons -b develop -cm $1",
                "projects":["teachstore-commons"]
            }
        ]
    }
    """



import git
import re, subprocess
from git import RemoteProgress
from pathlib import Path
import os, sys, ast, shutil
import configparser
import json
import datetime, time
from pymvngit.utils import Utils, Emoticons, Colors
from pymvngit.console import Console

class CloneProgress(RemoteProgress):
    def update(self, op_code, cur_count, max_count=None, msg=''):
        if msg:
            if Emoticons.isWindows():
               sys.stdout.write("\r  " + Colors.BLUE + " --> " + Colors.GREEN + msg + Colors.RESET)
            else:   
               sys.stdout.write("\r  " + Colors.BLUE + " --> " + Colors.GREEN + msg + Colors.RESET)


class PullProgress(RemoteProgress):
    def update(self, op_code, cur_count, max_count=None, msg=''):
        if msg:
           if Emoticons.isWindows():
               sys.stdout.write("\r  " + Colors.BLUE + " --> " + Colors.GREEN + msg + Colors.RESET)
           else:   
               sys.stdout.write("\r  " + Colors.BLUE + " --> " + Colors.GREEN + msg + Colors.RESET)

LABEL_LENGTH = 100

class PyGit:

    def __init__(self, command, arguments, projects):
        self._projects  = projects
        self._command   = command
        self._arguments = arguments
        self._executed  = False
        self.response   = None
        
    def execute(self):
        self._executed = True
        if "add" == self._command:
            self._add()
        elif "commit" == self._command:
            self._commit()    
        elif "status" == self._command:
            self._status()
        elif "push" == self._command:
            self._push()
        elif "pull" == self._command:
            self._pull()
        elif "tag" == self._command:
            self._tag()
        elif "clone" == self._command:
            self._clone()
        # custom command    
        elif "listTags".upper() == self._command.upper() or "listTag".upper() == self._command.upper():
            self._listTags()     
        else:
            Console.error("Git Command {} does not exist!".format(Colors.BIRED +  self._command + Colors.RESET))
            self._executed = False

    def _add(self):
        addAll = False
        if len(self._arguments) == 1 and self._arguments[0] == "-A":
           # -A to add all files 
           addAll = True
        else:
           # specific files to add 
           args = " ".join(self._arguments)
        msg  = ""
        for p in self._projects:
            added = False
            repo = git.Repo(p["path"])
            currentBranch = repo.active_branch
            current       = repo.create_head(currentBranch)
            current.checkout()
            if repo.index.diff(None) or repo.untracked_files:
               added = True
               origin = repo.remotes.origin
               
               msg  = Colors.BLUE + "Added " + Colors.GREEN + "repository {}".format(Colors.IBLUE +p["name"] + Colors.GREEN) 
               isOK = True
               try:
                   if addAll:
                      repo.git.add(A=True)
                   else:   
                      repo.git.add(args)
               except git.exc.GitCommandError as e:
                   isOK = False
                   msg += "\n  " + Colors.IRED + Emoticons.error() + " ERROR! \n  " + str(e)
               except:
                   isOK = False
                   msg += Colors.IRED + " Ops... " + Colors.IRED + str(sys.exc_info()[0]) 
                   msg = Console.addKoMsg(LABEL_LENGTH,msg)    
               if isOK:    
                   msg = Console.addOkMsg(LABEL_LENGTH,msg)    
               Console.printLine(msg)

            if not added:
               m = "Nothing to add for {repo}".\
                   format(repo=Colors.IBLUE +p["name"] + Colors.GREEN,iblue=Colors.IBLUE,green=Colors.GREEN)
               Console.printLine(Console.addOkMsg(LABEL_LENGTH,m))
               del m

    def _clone(self):
        if len(self._arguments) < 1:
           Console.error("The arguments \"{}\" doesn't seem to be a repository, or a list of them, to be cloned! Please, review it." \
           .format(Colors.IBLUE + " ".join(self._arguments) + Colors.GREEN))
           sys.exit()

        clonedRepos    = []
        repositories   = self._arguments[0]
        localPath      = self._arguments[1]
        branchName     = self._arguments[2]
        isRepoListFile = self._arguments[3]

        if not os.path.exists(localPath):
           Console.line(Emoticons.pin() +  Colors.GREEN + " Local path {} does not exist, creating..." \
                        .format(Colors.IBLUE+localPath+Colors.GREEN),False, False)
           os.makedirs(localPath)

        if isRepoListFile:
           fileName =  repositories[0]
           if not os.path.exists:
               Console.error("The file {} was not found!".format(Colors.IBLUE + fileName + Colors.GREEN))
               sys.exit()
               
           repositories = []    
           with open(fileName,'r') as f:
               for l in f:    
                   repositories.append(l.replace("\n","").strip())

        totalRepo = len(repositories)
        Console.printHeaderCommand("Clone Repositories, Total {}".format(Colors.IBLUE+str(totalRepo)+Colors.GREEN))   

        idx = 0
        for git_repo in repositories:
            idx += 1
            nameRepo        =  git_repo[git_repo.rfind("/")+1:].replace(".git","")
            localFolderName =  os.path.join(localPath,nameRepo)
            
            doIt         = True
            accessDenied = False
            isMaven      = False
            if os.path.exists(localFolderName):
               resp = input(Colors.GREEN + "{green}The folder {iblue}{folder}{green} already exists, delete and clone? [{igreen}Y{green}/{igreen}N{green}] ".\
                            format(igreen=Colors.IGREEN,green=Colors.GREEN,iblue=Colors.IBLUE,blue=Colors.BLUE,folder=localFolderName))
               if resp.strip().upper() == "Y":
                  try: 
                    shutil.rmtree(localFolderName)
                    doIt = True
                  except PermissionError as e:
                    msg = "Access Denied! Please remove the folder {} manually".format(Colors.IBLUE + localFolderName + Colors.GREEN)
                    print(Emoticons.error(),msg)
                    doIt = False
                    accessDenied = True
               else:
                  doIt = False 
                  
            if doIt:                  
               Console.line(Colors.GREEN + "Cloning repository" + " [{iblue}{idx:02d}{green} of {iblue}{total:02d}{green}] --> {iblue}{git_repo}{green}". \
                   format(idx=idx,iblue=Colors.IBLUE,git_repo=git_repo,green=Colors.GREEN,total=totalRepo),False,False)
               git.Repo.clone_from(git_repo, localFolderName, branch=branchName, progress=CloneProgress())           
               print(Colors.GREEN)                   

               for path in Path(localFolderName).rglob('pom.xml'):
                   isMaven = True
                   break

               clonedRepos.append({"name":nameRepo,"path":localFolderName,"git":"true","maven":isMaven})
            else:
               if not accessDenied:
                  Console.line(Colors.GREEN + "Not cloned!       " + " [{iblue}{idx:02d}{green} of {iblue}{total:02d}{green}] --> {iblue}{git_repo}{green} The folder Already exists!". \
                               format(idx=idx,iblue=Colors.IBLUE,git_repo=git_repo,green=Colors.GREEN,total=totalRepo),False,False)     

        Console.printFooter()
        if len(clonedRepos) > 0:  
           self.response = clonedRepos

    def _commit(self):
        if self._arguments[0].strip().lower() == "-m" and len(self._arguments) > 1:
           messageCommit = self._arguments[1] 
        else:
           Console.error("The arguments \"{}\" for git commit seems to be invalid, please check it!" \
           .format(Colors.IBLUE + " ".join(self._arguments) + Colors.GREEN))
           sys.exit()
        msg  = ""
        for p in self._projects:
            commited = False
            repo = git.Repo(p["path"])
            currentBranch = repo.active_branch
            current       = repo.create_head(currentBranch)
            current.checkout()
            if repo.index.diff("HEAD"):
               commited = True
               origin = repo.remotes.origin
               
               msg  = Colors.BLUE + "Commited " + Colors.GREEN + "repository {}".format(Colors.IBLUE +p["name"] + Colors.GREEN)
               isOK = True
               try:
                   repo.git.commit(m=messageCommit)
               except git.exc.GitCommandError as e:
                   isOK = False
                   msg += "\n  " + Colors.IRED + Emoticons.error() + " ERROR! \n  " + str(e)
               except:
                   isOK = False
                   msg += Colors.IRED + " Ops... " + Colors.IRED + str(sys.exc_info()[0]) 
                   msg = Console.addKoMsg(LABEL_LENGTH,msg)    
               if isOK:    
                   msg = Console.addOkMsg(LABEL_LENGTH,msg)    
               Console.printLine(msg)
   
            if not commited:
               m = "Nothing to commit for {repo}".\
                   format(repo=Colors.IBLUE +p["name"] + Colors.GREEN,iblue=Colors.IBLUE,green=Colors.GREEN)
               Console.printLine(Console.addOkMsg(LABEL_LENGTH,m))
               del m    

    def _status(self):
        msg = ""
        magnifier = "<< "  + Emoticons.magnifier()
        ok        = "Ok!"  + Emoticons.ok()
        for p in self._projects:
            nothingToCommit = True
            msg = Colors.BLUE + "Checking Status " + Colors.GREEN + "repository {}".format(Colors.IBLUE +p["name"] + Colors.GREEN)
            msg = Console.addToMsg(LABEL_LENGTH,msg,magnifier)

            repo  = git.Repo(p["path"])
            items = repo.index.diff(None)
            
            if len(items) > 0:
                nothingToCommit = False
                msg += "\n    " + Emoticons.pointRight() + Colors.GREEN + " Changed files:"
                idx_chg_files = 0
                for item in items:
                    idx_chg_files  += 1
                    msg += "\n       " + Colors.GREEN + "{:02d} ".format(idx_chg_files) + Colors.WHITE + ">>" + Colors.RESET + " " + item.a_path   
            files = repo.untracked_files
            if len(files) > 0:
                nothingToCommit = False
                msg += "\n    " + Emoticons.pointRight() + Colors.GREEN + " New added files:"
                idx_new_files = 0
                for file in files:
                    idx_new_files  += 1
                    msg += "\n       " + Colors.GREEN + "{:02d} ".format(idx_new_files) + Colors.WHITE + ">>" + Colors.RESET + " " + file

            if nothingToCommit:
               msg = msg.replace(magnifier,ok) 
               #msg = Console.addOkMsg(LABEL_LENGTH,msg)    
                
            Console.printLine(msg)

    def _push(self):
        msg = ""
        for p in self._projects:
            pushed = False
            repo = git.Repo(p["path"])
            currentBranch = repo.active_branch
            current       = repo.create_head(currentBranch)
            current.checkout()
            
            pushed = True
            origin = repo.remotes.origin
            
            pushingMsg = "{blue}Pushing {green}repository {iblue}{repo}{reset} {green}to branch origin/{branch}" \
                         .format(repo=p["name"],branch=currentBranch,iblue=Colors.IBLUE,green=Colors.GREEN,reset=Colors.RESET,blue=Colors.BLUE)
            pushingMsg = Console.addWaitMsg(LABEL_LENGTH,pushingMsg)
            Console.printLine(pushingMsg) 

            msg  = Colors.BLUE + "Pushed " + Colors.GREEN + "repository {} to branch origin/{}".format(Colors.IBLUE +p["name"] + Colors.GREEN,currentBranch) 
            isOK = True
            try:
                repo.git.push('--set-upstream', 'origin', current)
            except git.exc.GitCommandError as e:
                isOK = False
                msg += "\n  " + Colors.IRED + Emoticons.error() + " ERROR! \n  " + str(e)    
            except:
                isOK = False
                msg += Colors.RED + " Ops... " + Colors.IRED + str(sys.exc_info()[0]) 
                msg = Console.addKoMsg(LABEL_LENGTH,msg)    
            if isOK:    
                msg = Console.addOkMsg(LABEL_LENGTH,msg)
            Console.printLine(msg)

            if not pushed:
               print( Colors.GREEN + "  {iblue}--->{green} Ok! Nothing to push for {repo}".\
                      format(repo=Colors.IBLUE +p["name"] + Colors.GREEN,iblue=Colors.IBLUE,green=Colors.GREEN))
               

    def _pull(self):
        
        for p in self._projects:
            repo   = git.Repo(p["path"])
            origin = repo.remotes.origin
            msg  = Colors.BLUE + "Pulling " + Colors.GREEN + "repository \033[94m{}\033[32m from {}...". \
                   format(Colors.IBLUE + p["name"] + Colors.GREEN,origin.url)
            isOK = True
            try:
                origin.pull(progress=PullProgress())
            except git.exc.GitCommandError as e:
                isOK = False
                msg += "\n  " + Colors.IRED + Emoticons.error() + " ERROR! \n  " + str(e)     
            except:
                isOK = False
                msg += Colors.RED + " Ops... " + Colors.IRED + str(sys.exc_info()[0]) 
                msg = Console.addKoMsg(130,msg)    
            if isOK:    
               msg = Console.addOkMsg(130,msg)    
            Console.printLine(msg)    

    def _listTags(self):
        for p in self._projects:
            msg  = ""
            repo = git.Repo(p["path"])
            msg += Colors.BLUE + "Tags " + Colors.GREEN + "repository {}".format(Colors.IBLUE + p["name"] + Colors.RESET)
            for t in repo.tags:
                commitDate = time.strftime("%Y-%m-%d %H:%M %a", time.gmtime(t.commit.committed_date))
                tag        = Utils.alignRight(22,str(t))
                tagMsg     = t.tag.message
                tagCommit  = str(t.commit)
                msg += "\n       {green}>>{iblue} {tag} {blue}--> {green}{tagMsg}{reset},{green} {commitDate}{reset},{green} {tagCommit}" \
                    .format(green=Colors.GREEN,iblue=Colors.IBLUE,blue=Colors.BLUE,reset=Colors.RESET,tag=tag,tagMsg=tagMsg,commitDate=commitDate,tagCommit=tagCommit)
            Console.printLine(msg)


    def _tag(self):
        tag     = None
        message = None
        if self._arguments[0].strip().lower() == "-a" and len(self._arguments) > 1:
           # Annotated tag 
           tag = self._arguments[1] 
           if len(self._arguments) > 3 and self._arguments[2].strip().lower() == "-m":
              # Message for annoted tag 
              message = self._arguments[3]
           else:
              Console.error("The arguments \"{}\" for git tag seems to be invalid, please check it!" \
              .format(Colors.IBLUE + " ".join(self._arguments) + Colors.GREEN))
              sys.exit() 
        else:    
           # Not annoted tag, it is a simple tagging
           tag = self._arguments[0] 

        msg  = ""
        for p in self._projects:
            msgTagging = "  {iblue}---> Tagging{green} repository {iblue}{repo}{green} with tag {iblue}{tag}{green}". \
                        format(repo=p["name"],tag=tag,iblue=Colors.IBLUE,green=Colors.GREEN)
            print(msgTagging)  

            msg  = Colors.BLUE + "Tagged " + Colors.GREEN + "repository {iblue}{repo}{green} with tag {iblue}{tag}{green}". \
                        format(repo=p["name"],tag=tag,iblue=Colors.IBLUE,green=Colors.GREEN)
            isOK = True
            try:
                repo = git.Repo(p["path"])
                if message: 
                   new_tag = repo.create_tag(tag, message=message)
                else:
                   new_tag = repo.create_tag(tag) 
                repo.remotes.origin.push(new_tag)
            except git.exc.GitCommandError as e:
                isOK = False
                msg += "\n  " + Colors.IRED + Emoticons.error() + " ERROR! \n  " + str(e)    
            except:
                isOK = False
                msg += Colors.RED + " Ops... " + Colors.IRED + str(sys.exc_info()[0]) 
                msg = Console.addKoMsg(LABEL_LENGTH,msg)    
            if isOK:
               msg = Console.addOkMsg(LABEL_LENGTH,msg)
            Console.printLine(msg)




from pymvngit.utils import Utils,Emoticons,Colors
import os, sys

LINE_LENGTH = 150

class Console:

    @staticmethod
    def _separator():
        print(Colors.RESET + "+" + Colors.GREEN + "-".ljust(LINE_LENGTH,"-") + Colors.RESET + "+")
    
    @staticmethod
    def warning(msg):
        print("")
        print(" ",Emoticons.ops(),msg)
        print("")

    @staticmethod
    def error(msg):
        print("")
        print(" ",Emoticons.error(),msg)
        print("")

    @staticmethod
    def message(msg):
        print("  ",Colors.GREEN + msg + Colors.RESET)

    @staticmethod
    def title(msg,top=True,bottom=True):
        if top: Console._separator()
        print(" ",Emoticons.pin(),msg)
        if bottom: Console._separator()

    @staticmethod
    def title2(msg,top=True,bottom=True):
        if top: Console._separator()
        print(" ",Emoticons.blocks(),msg)
        if bottom: Console._separator()    

    @staticmethod
    def line(msg,top=True,bottom=True):
        if top: Console._separator()
        print(" ",msg)
        if bottom: Console._separator()    

    @staticmethod
    def printTable(header,prettyTable, tableArgs):
        Console.title(header)
        print(prettyTable.printMe(False, tableArgs))

    @staticmethod
    def printTable2(header,prettyTable,tableArgs):
        print(header)
        print(prettyTable.printMe(False, tableArgs))    

    @staticmethod
    def clear():
        if os.name == "nt":
           os.system('cls')
        else:   
           os.system('clear')

    @staticmethod
    def printLine(line,Arrow=True):
        arrow = Colors.IBLUE + "---> " if Arrow else ""
        print("  "  + arrow + Colors.GREEN + line)  
    
    @staticmethod
    def printHeaderCommand(command):
        #print(Colors.GREEN)
        print(Colors.RESET + "+" + Colors.GREEN + "-".ljust(LINE_LENGTH,"-") + Colors.RESET + "+" + Colors.GREEN)
        print("  " + Emoticons.pin() + "  " + Colors.GREEN + command + Colors.RESET)
        print(Colors.RESET + "+" + Colors.GREEN + "-".ljust(LINE_LENGTH,"-") + Colors.RESET + "+" + Colors.RESET)

    @staticmethod
    def printFooter():
        Console._separator()
        print("")
    
    @staticmethod
    def addOkMsg(length,msg):
        lenDiff = length - len(Utils.removeCharsColors(msg))
        return msg + ".".ljust(lenDiff,".") + ".Ok!" + Emoticons.ok()

    @staticmethod
    def addWaitMsg(length,msg):
        lenDiff = (length-2) - len(Utils.removeCharsColors(msg))
        return msg + ".".ljust(lenDiff,".") + ".Wait!" + Emoticons.waiting()

    @staticmethod
    def addKoMsg(length,msg):
        lenDiff = length - len(Utils.removeCharsColors(msg))
        return msg + ".".ljust(lenDiff," ") + ".KO!" + Emoticons.error()
    
    @staticmethod
    def addToMsg(length,msg,texto):
        lenDiff = length - len(Utils.removeCharsColors(msg))
        return msg + ".".ljust(lenDiff,".") + "." + texto




import json, os, sys, datetime
from io import StringIO
from pygments import highlight, lexers, formatters

class Utils:
    
    @staticmethod
    def encodeString(str):
        bytes        = str.encode("utf-8")
        base64_bytes =  base64.b64encode(bytes)
        return base64_bytes.decode("utf-8")

    @staticmethod
    def decodeString(str):
        bytes        = str.encode("utf-8")
        base64_bytes =  base64.b64decode(bytes)
        return base64_bytes.decode("utf-8")

    @staticmethod
    def stringToJSON(str):
        return json.loads(str)

    @staticmethod
    def dictToJSON(dictObj):
        strObj = StringIO()
        json.dump(dictObj, strObj, default=str, indent=4, sort_keys=True)
        return strObj.getvalue()

    @staticmethod
    def printJSON(jsonObject):
        colorful_json = highlight(jsonObject, lexers.JsonLexer(), formatters.TerminalFormatter())
        print(colorful_json)

    @staticmethod
    def alignRight(length,vlr):
        return vlr + " ".ljust(length-len(vlr)," ")    

    @staticmethod
    def removeCharsColors(text):
        if isinstance(text, str):
           text = text.replace(Colors.UNDERLINE,"") \
                      .replace(Colors.RESET,"") \
                      .replace(Colors.BLACK,"") \
                      .replace(Colors.RED,"") \
                      .replace(Colors.GREEN,"") \
                      .replace(Colors.YELLOW,"") \
                      .replace(Colors.BLUE,"") \
                      .replace(Colors.MAGENTA,"") \
                      .replace(Colors.CYAN,"") \
                      .replace(Colors.WHITE,"") \
                      .replace(Colors.BBLACK,"") \
                      .replace(Colors.BRED,"") \
                      .replace(Colors.BGREEN,"") \
                      .replace(Colors.BYELLOW,"") \
                      .replace(Colors.BBLUE,"") \
                      .replace(Colors.BMAGENTA,"") \
                      .replace(Colors.BCYAN,"") \
                      .replace(Colors.BWHITE,"") \
                      .replace(Colors.BG_BLACK,"") \
                      .replace(Colors.BG_RED,"") \
                      .replace(Colors.BG_GREEN,"") \
                      .replace(Colors.BG_YELLOW,"") \
                      .replace(Colors.BG_BLUE,"") \
                      .replace(Colors.BG_PURPLE,"") \
                      .replace(Colors.BG_CYAN,"") \
                      .replace(Colors.BG_WHITE,"") \
                      .replace(Colors.IBLACK,"") \
                      .replace(Colors.IRED,"") \
                      .replace(Colors.IGREEN,"") \
                      .replace(Colors.IYELLOW,"") \
                      .replace(Colors.IBLUE,"") \
                      .replace(Colors.IMAGENTA,"") \
                      .replace(Colors.ICYAN,"") \
                      .replace(Colors.IWWHITE,"") \
                      .replace(Colors.BIBLACK,"") \
                      .replace(Colors.BIRED,"") \
                      .replace(Colors.BIGREEN,"") \
                      .replace(Colors.BIYELLOW,"") \
                      .replace(Colors.BIBLUE,"") \
                      .replace(Colors.BIPURPLE,"") \
                      .replace(Colors.BICYAN,"") \
                      .replace(Colors.BIWHITE,"") \
                      .replace(Colors.On_IBLACK,"") \
                      .replace(Colors.On_IRED,"") \
                      .replace(Colors.On_IGREEN,"") \
                      .replace(Colors.On_IYELLOW,"") \
                      .replace(Colors.On_IBLUE,"") \
                      .replace(Colors.On_IPURPLE,"") \
                      .replace(Colors.On_ICYAN,"") \
                      .replace(Colors.On_IWHITE,"")
        return text    


class Colors:
    
    UNDERLINE = '\033[4m'
    RESET     = '\033[0m'

    # Regular Colors
    BLACK     = '\033[0;30m'
    RED       = '\033[0;31m'
    GREEN     = '\033[0;32m'
    YELLOW    = '\033[0;33m'
    BLUE      = '\033[0;34m'
    MAGENTA   = '\033[0;35m'
    CYAN      = '\033[0;36m'
    WHITE     = '\033[0;37m'

    # Bold
    BBLACK    ="\033[1;30m"
    BRED      ="\033[1;31m"
    BGREEN    ="\033[1;32m"
    BYELLOW   ="\033[1;33m"
    BBLUE     ="\033[1;34m"
    BMAGENTA  ="\033[1;35m"
    BCYAN     ="\033[1;36m"
    BWHITE    ="\033[1;37m"

    # Background Colors
    BG_BLACK  = '\033[40m'
    BG_RED    = '\033[41m'
    BG_GREEN  = '\033[42m'
    BG_YELLOW = '\033[43m'
    BG_BLUE   = '\033[44m'
    BG_PURPLE = '\033[45m'
    BG_CYAN   = '\033[46m'
    BG_WHITE  = '\033[47m'

    # High Intensty
    IBLACK    ="\033[0;90m"
    IRED      ="\033[0;91m"
    IGREEN    ="\033[0;92m"
    IYELLOW   ="\033[0;93m"
    IBLUE     ="\033[0;94m"
    IMAGENTA  ="\033[0;95m"
    ICYAN     ="\033[0;96m"
    IWWHITE   ="\033[0;97m"

    # Bold High Intensty
    BIBLACK   ="\033[1;90m"
    BIRED     ="\033[1;91m"
    BIGREEN   ="\033[1;92m"
    BIYELLOW  ="\033[1;93m"
    BIBLUE    ="\033[1;94m"
    BIPURPLE  ="\033[1;95m"
    BICYAN    ="\033[1;96m"
    BIWHITE   ="\033[1;97m"

    # High Intensty backgrounds
    On_IBLACK ="\033[0;100m"
    On_IRED   ="\033[0;101m"
    On_IGREEN ="\033[0;102m"
    On_IYELLOW="\033[0;103m"
    On_IBLUE  ="\033[0;104m"
    On_IPURPLE="\033[0;105m"
    On_ICYAN  ="\033[0;106m"
    On_IWHITE ="\033[0;107m"

class Emoticons:

    _PROMPT      = ['⛅','⛅']
    _WAITING     = ['💤💤💤','☕☕☕']
    _SEE_YA      = ['👋','✌']
    _ERROR       = ['❌💣','❌☠']
    _TOOL        = ['🔧','⚒']
    _THUMBS_UP   = ['👍','✔']
    _POINT_RIGHT = ['👉','❖']
    _WINK        = ['😉','☻']
    _OPS         = ['😕','☹']
    _PIN         = ['📌','✎']
    _ENV         = ['📝','✍']
    _TIME        = ['🕘','☕']
    _WAIT_DISTR  = ['🍺','♨']
    _WAIT_DISTR2 = ['🍼','⚾']
    _MAGNIFIER   = ['🔍','☌']
    _BLOCKS      = ['📦','❒']
    _REDMARK     = ['🔴','⚫']
    _UPLOAD      = ['📤','✈']
    _UPLOAD_PART = ['🔹','➩']
    _FOLDER      = ['🔹','➩']
    _OK          = ['✅','✅']
    _IMGS        = [
                     ['🕒','🕓','🕔','🕕','🕖','🕗','🕘','🕙','🕚','🕛','🕐','🕑'],
                     ['☰','☱','☲','☴','☵','☶','☷','☶','☴']
                   ]

    @staticmethod
    def isWindows():
        return 1 if os.name == "nt" else 0
    
    @staticmethod
    def prompt():
        return Emoticons._PROMPT[Emoticons.isWindows()]
    @staticmethod
    def waiting():
        return Emoticons._WAITING[Emoticons.isWindows()]    
    @staticmethod
    def seeYa():
        return Emoticons._SEE_YA[Emoticons.isWindows()]
    @staticmethod
    def error():
        return Emoticons._ERROR[Emoticons.isWindows()]
    @staticmethod
    def tool():
        return Emoticons._TOOL[Emoticons.isWindows()]
    @staticmethod
    def thumbsUp():
        return Emoticons._THUMBS_UP[Emoticons.isWindows()] 
    @staticmethod
    def pointRight():
        return Emoticons._POINT_RIGHT[Emoticons.isWindows()]
    @staticmethod
    def wink():
        return Emoticons._WINK[Emoticons.isWindows()]
    @staticmethod
    def pin():
        return Emoticons._PIN[Emoticons.isWindows()]
    @staticmethod
    def env():
        return Emoticons._ENV[Emoticons.isWindows()]
    @staticmethod
    def time():
        return Emoticons._TIME[Emoticons.isWindows()]
    @staticmethod
    def waitDistract():
        return Emoticons._WAIT_DISTR[Emoticons.isWindows()]
    @staticmethod
    def waitDistract2():
        return Emoticons._WAIT_DISTR2[Emoticons.isWindows()]
    @staticmethod
    def ops():
        return Emoticons._OPS[Emoticons.isWindows()]
    @staticmethod
    def magnifier():
        return Emoticons._MAGNIFIER[Emoticons.isWindows()]
    @staticmethod
    def ok():
        return Emoticons._OK[Emoticons.isWindows()]    
    @staticmethod
    def redMark():
        return Emoticons._REDMARK[Emoticons.isWindows()]
    @staticmethod
    def blocks():
        return Emoticons._BLOCKS[Emoticons.isWindows()]        





import pathlib
from setuptools import setup,find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# 
setup(
    name="pymvngit",
    version="0.1.14",
    description="Tool to manage a group of Git/Maven projects.",
    long_description=README,
    long_description_content_type="text/markdown",
    #url="",
    author="Ualter Otoni Pereira",
    author_email="ualter.junior@gmail.com",
    keywords = ['maven','git','tool','java','microservices','lerna'],
    license="MIT",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=["","pymvngit"],
    include_package_data=True,
    install_requires=[            
        'tinydb','Pygments','clipboard','gitpython'
    ],
    entry_points={
        "console_scripts": [
            "pymvngit=pymvngit.pymvngit:main",
        ]
    },
)